import os
from diaspora_logger import DiasporaLogger


def on_send_success(record_metadata):
    print(
        f'Message sent to {record_metadata.topic}:{record_metadata.partition} {record_metadata.offset}')


def run_producer_example(topic):
    # Read the refresh token from the environment variable
    refresh_token = os.getenv('DIASPORA_REFRESH')

    if not refresh_token:
        raise ValueError("Environment variable DIASPORA_REFRESH not set")

    kafka_logger = DiasporaLogger(
        bootstrap_servers=["52.200.217.146:9093", "54.210.46.108:9094"],
        refresh_token=refresh_token,
    )

    future = kafka_logger.send(topic, {'message': 'Synchronous message'})
    result = future.get(timeout=10)
    print(result)

    future = kafka_logger.send(topic, {'message': 'Asynchronous message'})
    future.add_callback(on_send_success)



# Content of logger.py
import json
import urllib.parse
import http.client
from kafka import KafkaProducer
from kafka.oauth.abstract import AbstractTokenProvider
import logging

# logging.basicConfig(
#     level=logging.INFO,
#     format='%(asctime)s %(levelname)s:%(name)s:%(message)s'
# )

# Create a named logger
logger = logging.getLogger('diaspora')


class GlobusAuthTokenProvider(AbstractTokenProvider):
    def __init__(self, refresh_token, **config):
        super().__init__(**config)
        self.refresh_token = refresh_token

    def post_to_globus(self, refresh_token):
        conn = http.client.HTTPSConnection("auth.globus.org")
        params = urllib.parse.urlencode({
            "refresh_token": refresh_token,
            "grant_type": "refresh_token",
            "client_id": "c5d4fab4-7f0d-422e-b0c8-5c74329b52fe",
        })
        headers = {"Content-Type": "application/x-www-form-urlencoded"}
        conn.request("POST", "/v2/oauth2/token", params, headers)

        response = conn.getresponse()
        response_body = response.read().decode()
        conn.close()

        if response.status == 200:
            return json.loads(response_body)
        else:
            print(f"Failed to retrieve token: {response_body}")
            return None

    def token(self):
        token = self.post_to_globus(self.refresh_token)
        logger.info("client-side new token generated:")
        logger.info(token)
        return token['access_token']


class DiasporaLogger:
    def __init__(self, bootstrap_servers, refresh_token):
        self._provider = GlobusAuthTokenProvider(refresh_token)
        self._producer = KafkaProducer(
            bootstrap_servers=bootstrap_servers,
            sasl_oauth_token_provider=self._provider,
            security_protocol="SASL_PLAINTEXT",
            sasl_mechanism="OAUTHBEARER",
            api_version=(3, 5, 1),
            value_serializer=lambda v: json.dumps(v).encode('utf-8')
        )

    def __getattr__(self, name):
        # Delegate attribute access to self._producer
        return getattr(self._producer, name)



import os
from diaspora_logger import GlobusAuthTokenProvider
from kafka import KafkaConsumer


def run_consumer_example(topic, group_id):
    # Obtain the refresh token from the environment variable
    refresh_token = os.getenv('DIASPORA_REFRESH')

    if not refresh_token:
        raise ValueError("Environment variable DIASPORA_REFRESH not set")

    bootstrap_servers = ["52.200.217.146:9093"]
    security_protocol = "SASL_PLAINTEXT"
    sasl_mechanism = "OAUTHBEARER"
    provider = GlobusAuthTokenProvider(refresh_token)
    consumer = KafkaConsumer(topic,
                             group_id=group_id,
                             bootstrap_servers=bootstrap_servers,
                             security_protocol=security_protocol,
                             sasl_mechanism=sasl_mechanism,
                             sasl_oauth_token_provider=provider,
                             api_version=(3, 5, 1))
    for msg in consumer:
        print(msg)



import webbrowser
import base64
import hashlib
import os
import http.client
import urllib.parse
import json


def generate_code_verifier_and_challenge():
    code_verifier = base64.urlsafe_b64encode(
        os.urandom(32)).decode("utf-8").rstrip("=")
    hashed_verifier = hashlib.sha256(code_verifier.encode("utf-8")).digest()
    code_challenge = base64.urlsafe_b64encode(
        hashed_verifier).decode("utf-8").rstrip("=")
    return code_verifier, code_challenge


def build_authorization_url(code_challenge):
    params = {
        "client_id": "c5d4fab4-7f0d-422e-b0c8-5c74329b52fe",
        "redirect_uri": "https://auth.globus.org/v2/web/auth-code",
        "scope": "openid email profile offline_access https://auth.globus.org/scopes/2b9d2f5c-fa32-45b5-875b-b24cd343b917/action_all",
        "state": "_default",
        "response_type": "code",
        "code_challenge": code_challenge,
        "code_challenge_method": "S256"
    }
    param_str = urllib.parse.urlencode(params)
    return f"https://auth.globus.org/v2/oauth2/authorize?{param_str}"


def request_token(code, code_verifier):
    conn = http.client.HTTPSConnection("auth.globus.org")
    params = urllib.parse.urlencode({
        "client_id": "c5d4fab4-7f0d-422e-b0c8-5c74329b52fe",
        "redirect_uri": "https://auth.globus.org/v2/web/auth-code",
        "grant_type": "authorization_code",
        "code": code,
        "code_verifier": code_verifier
    })
    headers = {"Content-Type": "application/x-www-form-urlencoded"}
    conn.request("POST", "/v2/oauth2/token", body=params, headers=headers)
    response = conn.getresponse()
    response_body = response.read().decode()
    conn.close()

    if response.status == 200:
        return json.loads(response_body)
    else:
        print(f"Failed to retrieve access token: {response_body}")


def decode_jwt(jwt):
    payload = jwt.split('.')[1]
    payload_padding = len(payload) % 4
    payload += "="*payload_padding
    payload_decoded = json.loads(
        base64.urlsafe_b64decode(payload).decode('utf-8'))

    return payload_decoded


def claim_topics(sub):
    conn = http.client.HTTPConnection("52.200.217.146:9090")
    conn.request("POST", f"/acl?sub={sub}")
    print(f"POSTing to the ACL endpoint with /acl?sub={sub}")

    response = conn.getresponse()
    response_body = response.read().decode()
    conn.close()

    if response.status == 200:
        print(f"Successful registration: {response_body}")
    else:
        print(f"Failed to register: {response_body}")


def request_token_workflow(save_for_java=False):
    code_verifier, code_challenge = generate_code_verifier_and_challenge()
    authorization_url = build_authorization_url(code_challenge)
    print(
        f"Please visit the following URL to authorize the application:\n{authorization_url}")
    webbrowser.open(authorization_url)
    authorization_code = input("Paste the authorization code here: ").strip()
    token = request_token(authorization_code, code_verifier)
    if token:
        access_token = token['other_tokens'][0]['access_token']
        refresh_token = token['other_tokens'][0]['refresh_token']
        payload = decode_jwt(token['id_token'])
        preferred_username = payload['preferred_username'].split("@")[0]
        print("="*32)
        print("Subject claim:\t", payload['sub'])
        print("Principal name:\t", preferred_username)
        print("Access token:\t", access_token)
        print("Refresh token:\t", refresh_token)
        print("Py clients:\t",
              f"export DIASPORA_REFRESH={refresh_token}")
        if save_for_java:
            file_name = f"{preferred_username}.properties"
            print("Java clients: \t",
              f"Connection properties are saved to {file_name}.")
            with open(file_name, "w") as f:
                f.write("security.protocol=SASL_PLAINTEXT\n")
                f.write("sasl.mechanism=OAUTHBEARER\n")
                f.write("sasl.login.callback.handler.class=com.oauth2.security.oauthbearer.OAuthAuthenticateLoginCallbackHandler\n")
                f.write("sasl.jaas.config=org.apache.kafka.common.security.oauthbearer.OAuthBearerLoginModule required \\\n")
                f.write(f"     OAUTH_REFRESH_TOKEN='{refresh_token}';")
        print("="*32)
        
if __name__ == "__main__":
    request_token_workflow()



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='diaspora_logger',
    version='0.0.8',
    description='A Python logging library for sending logs to Diaspora Streams (Kafka)',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=find_packages(),
    include_package_data=True,  # This flag is used to include non-code files
    license='LICENSE',  # Specify the path to your LICENSE file
    url='https://github.com/globus-labs/diaspora_logger',
    install_requires=[
        'kafka-python',
    ],
)
